/*********************************************************************
* DETAB.C - This program is a DOS filter that replaces tab stops in  *
*   the input stream with spaces based on a uniform tab stop size.   *
*                                                                    *
* To compile: CL DETAB.C                                             *
*********************************************************************/
#include <stdio.h>     /* fgetc(), fputc(), EOF                     */

#define LF    0x0a     /* Line feed character                       */
#define CR    0x0d     /* Carriage return character                 */
#define TAB   0x09     /* Tab stop character                        */
#define SPACE 0x20     /* Space character                           */

/*----Macro to output character and advance position indicator------*/
#define StreamOut(Ch)  { fputc( Ch, stdout ); ++Pos; }

/*********************************************************************
* main() - Main body of program                                      *
*********************************************************************/
void main( int argc, char **argv )
    {
    int Ch;                      /* Temporary character/int storage */
    int Pos = 0;                 /* Indicates current line position */
    int TabSize = 8;             /* Interval between tab stops      */

    /*----Get tab size from command-line----------------------------*/
    if ( argc > 1 )
        if ( (Ch = atoi(argv[1])) <= 79 )
            TabSize = Ch;

    /*----Process until end-of-file encountered---------------------*/
    while ( (Ch = fgetc( stdin )) != EOF )
        {
        switch( Ch )
            {
            case CR :  /* Handle end of line CR/LF or just LF       */
            case LF :
                /*----Output character and reset position-----------*/
                StreamOut( Ch );
                Pos = 0;
                break;
            case TAB : /* Expand tab into space characters          */
                /*----Handle tab stop at tab position---------------*/
                if ( Pos % TabSize == 0 )
                    StreamOut( SPACE );
                /*----Insert spaces until we reach next tab stop----*/
                while ( Pos % TabSize > 0 ) 
                    StreamOut( SPACE );
                break;
            default :  /* All characters other than CR, LF, and TAB */
                /*----Output character and advance position---------*/
                StreamOut( Ch );
                break;
            }
        }
    }
